/*
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

@file:Suppress("DEPRECATION")

package com.raywenderlich.kanime.activities

import android.app.ProgressDialog
import android.os.Bundle
import android.os.Handler
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import com.raywenderlich.kanime.R
import com.raywenderlich.kanime.adapters.AnimeAdapter
import com.raywenderlich.kanime.adapters.OnAnimeClickListener
import com.raywenderlich.kanime.data.getAnimes
import com.raywenderlich.kanime.model.Anime
import org.jetbrains.anko.*

class MainActivity : AppCompatActivity(), OnAnimeClickListener, AnkoLogger {

  companion object {
    private const val COLUMNS_NUMBER = 2
  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(R.layout.activity_main)

    val animesList = findViewById<RecyclerView>(R.id.animes)
    animesList.adapter = AnimeAdapter(getAnimes(), this)
    animesList.layoutManager = GridLayoutManager(this, COLUMNS_NUMBER)

    logInfo()
    logVerbose()
    logDebug()
    logWarn()
    logError()
    logWTF()
  }

  override fun onClick(anime: Anime) {
    val dialog = showLoadingDialog(
        getString(R.string.please_wait), getString(R.string.template_downloading_inf, anime.name))

    //Wait for 2 seconds
    waitABit {
      dialog.dismiss()
      openDetailActivity(anime)
    }
  }


  override fun onOptionsItemSelected(item: MenuItem): Boolean {

    return when (item.itemId) {
      R.id.info -> {
        openAboutActivity()
        true
      }

      else -> super.onOptionsItemSelected(item)
    }
  }

  override fun onCreateOptionsMenu(menu: Menu): Boolean {
    val inflater = menuInflater
    inflater.inflate(R.menu.main, menu)
    return true
  }

  private fun waitABit(done: () -> Unit) {
    val handler = Handler()
    handler.postDelayed({ done() }, 2000)
  }

  private fun openDetailActivity(anime: Anime) {
    startActivity<AnimeDetailActivity>(
        "TITLE_KEY" to anime.name,
        "DESCRIPTION_KEY" to anime.description,
        "IMDB_LINK_KEY" to anime.imdbLink,
        "IMAGE_KEY" to anime.imageDrawable
    )
  }

  private fun openAboutActivity() {
    startActivity<AboutActivity>()
  }

  private fun showLoadingDialog(message: String, title: String): ProgressDialog {
    val dialog = ProgressDialog(this)
    dialog.setMessage(message)
    dialog.setTitle(title)
    dialog.show()
    return dialog
  }

  private fun logInfo() {
    info("Log Info") //Will log I/MainActivity: Log Info

    info {
      "Log Info" //Will log I/MainActivity: Log Info
    }
  }

  private fun logVerbose() {
    verbose("Log Verbose") //Will log V/MainActivity: Log Verbose

    verbose {
      "Log Verbose" //Will log V/MainActivity: Log Verbose
    }
  }

  private fun logDebug() {
    debug("Log Debug") //Will log D/MainActivity: Log Debug

    debug {
      "Log Debug" //Will log D/MainActivity: Log Debug
    }
  }

  private fun logWarn() {
    warn("Log Warn") //Will log W/MainActivity: Log Warn

    warn {
      "Log Warn" //Will log W/MainActivity: Log Warn
    }
  }

  private fun logError() {
    error("Log Error") //Will log E/MainActivity: Log Error
  }

  private fun logWTF() {
    // What a Terrible Failure
    wtf("Log WTF") //  //Will log E/MainActivity: Log WTF
  }
}
