/*
 * Copyright (c) 2018 Razeware LLC
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * Notwithstanding the foregoing, you may not use, copy, modify, merge, publish,
 * distribute, sublicense, create a derivative work, and/or sell copies of the
 * Software in any work that is designed, intended, or marketed for pedagogical or
 * instructional purposes related to programming, coding, application development,
 * or information technology.  Permission for such use, copying, modification,
 * merger, publication, distribution, sublicensing, creation of derivative works,
 * or sale is expressly withheld.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

@file:Suppress("DEPRECATION")

package com.raywenderlich.kanime.activities

import android.app.ProgressDialog
import android.content.Intent
import android.os.Bundle
import android.os.Handler
import android.support.v7.app.AppCompatActivity
import android.support.v7.widget.GridLayoutManager
import android.support.v7.widget.RecyclerView
import android.util.Log
import android.view.Menu
import android.view.MenuItem
import com.raywenderlich.kanime.R
import com.raywenderlich.kanime.adapters.AnimeAdapter
import com.raywenderlich.kanime.adapters.OnAnimeClickListener
import com.raywenderlich.kanime.data.getAnimes
import com.raywenderlich.kanime.model.Anime

class MainActivity : AppCompatActivity(), OnAnimeClickListener {

  companion object {
    private const val COLUMNS_NUMBER = 2
  }

  override fun onCreate(savedInstanceState: Bundle?) {
    super.onCreate(savedInstanceState)
    setContentView(R.layout.activity_main)

    val animesList = findViewById<RecyclerView>(R.id.animes)
    animesList.adapter = AnimeAdapter(getAnimes(), this)
    animesList.layoutManager = GridLayoutManager(this, COLUMNS_NUMBER)

    logInfo()
    logVerbose()
    logDebug()
    logWarn()
    logError()
    logWTF()
  }

  override fun onClick(anime: Anime) {
    val dialog = showLoadingDialog(
        getString(R.string.please_wait), getString(R.string.template_downloading_inf, anime.name))

    //Wait for 2 seconds
    waitABit {
      dialog.dismiss()
      openDetailActivity(anime)
    }
  }


  override fun onOptionsItemSelected(item: MenuItem): Boolean {

    return when (item.itemId) {
      R.id.info -> {
        openAboutActivity()
        true
      }

      else -> super.onOptionsItemSelected(item)
    }
  }

  override fun onCreateOptionsMenu(menu: Menu): Boolean {
    val inflater = menuInflater
    inflater.inflate(R.menu.main, menu)
    return true
  }

  private fun waitABit(done: () -> Unit) {
    val handler = Handler()
    handler.postDelayed({ done() }, 2000)
  }

  private fun openDetailActivity(anime: Anime) {
    //1. Create the intent and specify the target activity
    val intent = Intent(this, AnimeDetailActivity::class.java)
    //2. Add anime details
    intent.putExtra("TITLE_KEY", anime.name)
    intent.putExtra("DESCRIPTION_KEY", anime.description)
    intent.putExtra("IMDB_LINK_KEY", anime.imdbLink)
    intent.putExtra("IMAGE_KEY", anime.imageDrawable)
    //3. open the detail activity
    startActivity(intent)
  }

  private fun openAboutActivity() {
    //1. Create the intent and specify the target activity
    val intent = Intent(this, AboutActivity::class.java)
    //2. open the detail activity
    startActivity(intent)
  }

  private fun showLoadingDialog(message: String, title: String): ProgressDialog {
    val dialog = ProgressDialog(this)
    dialog.setMessage(message)
    dialog.setTitle(title)
    dialog.show()
    return dialog
  }

  private fun logInfo() {
    val tag = "MainActivity"
    val message = "Log Info"

    if (Log.isLoggable(tag, Log.INFO))
      Log.i(tag, message)
    //Will log I/MainActivity: Log Info
  }


  private fun logVerbose() {
    val tag = "MainActivity"
    val message = "Log Verbose"

    if (Log.isLoggable(tag, Log.VERBOSE))
      Log.v(tag, message)
    //Will log I/MainActivity: Log Verbose
  }

  private fun logDebug() {
    val tag = "MainActivity"
    val message = "Log Debug"

    if (Log.isLoggable(tag, Log.DEBUG))
      Log.d(tag, message)
    //Will log I/MainActivity: Log Debug
  }

  private fun logWarn() {
    val tag = "MainActivity"
    val message = "Log Warn"

    if (Log.isLoggable(tag, Log.WARN))
      Log.w(tag, message)
    //Will log I/MainActivity: Log Warn

  }

  private fun logError() {
    val tag = "MainActivity"
    val message = "Log Error"

    if (Log.isLoggable(tag, Log.ERROR))
      Log.e(tag, message)
    //Will log I/MainActivity: Log Error
  }

  private fun logWTF() {
    val tag = "MainActivity"
    val message = "Log WTF"

    Log.wtf(tag, message)
    //Will log I/MainActivity: Log WTF
  }
}
